@@ -12,8 +12,13 @@ import (
 	"time"
 
 	pkgerrors "github.com/pkg/errors"
+	"github.com/powerman/deepequal"
+	"google.golang.org/grpc/status"
+	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/reflect/protoreflect"
 )
 
+//nolint:gochecknoglobals // Const.
 var (
 	typString  = reflect.TypeOf("")
 	typBytes   = reflect.TypeOf([]byte(nil))
@@ -24,6 +29,7 @@ var (
 type C struct {
 	*testing.T
 	todo bool
+	must bool
 }
 
 const (
@@ -44,7 +50,7 @@ func (t *C) Parallel() {
 	}
 }
 
-// T create and return new *C, which wraps given tt and supposed to be
+// T creates and returns new *C, which wraps given tt and supposed to be
 // used inplace of it, providing you with access to many useful helpers in
 // addition to standard methods of *testing.T.
 //
@@ -55,11 +61,11 @@ func (t *C) Parallel() {
 //		t := check.T(tt)
 //		// use only t in test and don't touch tt anymore
 //	}
-func T(tt *testing.T) *C {
+func T(tt *testing.T) *C { //nolint:thelper // With check we name it tt!
 	return &C{T: tt}
 }
 
-// TODO create and return new *C, which have only one difference from
+// TODO creates and returns new *C, which have only one difference from
 // original one: every passing check is now handled as failed and vice
 // versa (this doesn't affect boolean value returned by check).
 // You can continue using both old and new *C at same time.
@@ -92,7 +98,16 @@ func T(tt *testing.T) *C {
 //		...
 //	}
 func (t *C) TODO() *C {
-	return &C{T: t.T, todo: true}
+	return &C{T: t.T, todo: true, must: t.must}
+}
+
+// MustAll creates and returns new *C, which have only one difference from
+// original one: every failed check will interrupt test using t.FailNow.
+// You can continue using both old and new *C at same time.
+//
+// This provides an easy way to turn all checks into assertion.
+func (t *C) MustAll() *C {
+	return &C{T: t.T, todo: t.todo, must: true}
 }
 
 func (t *C) pass() {
@@ -119,7 +134,7 @@ func (t *C) fail() {
 	stats[t.T].failed.value++
 }
 
-func (t *C) report(ok bool, msg []interface{}, checker string, name []string, args []interface{}) bool {
+func (t *C) report(ok bool, msg []interface{}, checker string, name []string, args []interface{}) bool { //nolint:revive // False positive.
 	t.Helper()
 
 	if ok != t.todo {
@@ -156,7 +171,7 @@ func (t *C) report(ok bool, msg []interface{}, checker string, name []string, ar
 	failureLong := failure.String()
 
 	wantDiff := len(dump) == 2 && name[0] == nameActual && name[1] == nameExpected
-	if wantDiff {
+	if wantDiff { //nolint:nestif // No idea how to simplify.
 		if reportToGoConvey(dump[0].String(), dump[1].String(), failureShort) == nil {
 			t.Fail()
 		} else {
@@ -172,6 +187,10 @@ func (t *C) report(ok bool, msg []interface{}, checker string, name []string, ar
 	}
 
 	t.fail()
+
+	if t.must {
+		t.FailNow()
+	}
 	return ok
 }
 
@@ -238,10 +257,10 @@ func (t *C) report3(actual, expected1, expected2 interface{}, msg []interface{},
 
 // Must interrupt test using t.FailNow if called with false value.
 //
-// This provides easy way to turn any check into assertion:
+// This provides an easy way to turn any check into assertion:
 //
 //   t.Must(t.Nil(err))
-func (t *C) Must(continueTest bool, msg ...interface{}) {
+func (t *C) Must(continueTest bool, msg ...interface{}) { //nolint:revive // False positive.
 	t.Helper()
 	t.report0(msg, continueTest)
 	if !continueTest {
@@ -299,7 +318,8 @@ func (t *C) should1(f ShouldFunc1, args ...interface{}) bool {
 
 func (t *C) should2(f ShouldFunc2, args ...interface{}) bool {
 	t.Helper()
-	if len(args) < 2 {
+	const minArgs = 2
+	if len(args) < minArgs {
 		panic("not enough params for " + funcName(f))
 	}
 	actual, expected, msg := args[0], args[1], args[2:]
@@ -353,6 +373,13 @@ func isNil(actual interface{}) bool {
 		return actual == nil
 	case reflect.Chan, reflect.Func, reflect.Map, reflect.Ptr, reflect.Slice:
 		return val.IsNil()
+	case reflect.Uintptr, reflect.UnsafePointer: // Subtle cases documented above.
+	case reflect.Interface: // ???
+	// Can't be nil:
+	case reflect.Struct, reflect.Array, reflect.Bool, reflect.String:
+	case reflect.Complex128, reflect.Complex64, reflect.Float32, reflect.Float64:
+	case reflect.Int, reflect.Int16, reflect.Int32, reflect.Int64, reflect.Int8:
+	case reflect.Uint, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uint8:
 	}
 	return false
 }
@@ -366,6 +393,14 @@ func (t *C) NotNil(actual interface{}, msg ...interface{}) bool {
 		!isNil(actual))
 }
 
+// Error is equivalent to Log followed by Fail.
+//
+// It is like t.Errorf with TODO() and statistics support.
+func (t *C) Error(msg ...interface{}) {
+	t.Helper()
+	t.report0(msg, false)
+}
+
 // True checks for cond == true.
 //
 // This can be useful to use your own custom checks, but this way you
@@ -444,17 +479,35 @@ func (t *C) NotBytesEqual(actual, expected []byte, msg ...interface{}) bool {
 }
 
 // DeepEqual checks for reflect.DeepEqual(actual, expected).
+// It will also use Equal method for types which implements it
+// (e.g. time.Time, decimal.Decimal, etc.).
+// It will use proto.Equal for protobuf messages.
 func (t *C) DeepEqual(actual, expected interface{}, msg ...interface{}) bool {
 	t.Helper()
+	protoActual, proto1 := actual.(protoreflect.ProtoMessage)
+	protoExpected, proto2 := expected.(protoreflect.ProtoMessage)
+	if proto1 && proto2 {
+		return t.report2(actual, expected, msg,
+			proto.Equal(protoActual, protoExpected))
+	}
 	return t.report2(actual, expected, msg,
-		reflect.DeepEqual(actual, expected))
+		deepequal.DeepEqual(actual, expected))
 }
 
 // NotDeepEqual checks for !reflect.DeepEqual(actual, expected).
+// It will also use Equal method for types which implements it
+// (e.g. time.Time, decimal.Decimal, etc.).
+// It will use proto.Equal for protobuf messages.
 func (t *C) NotDeepEqual(actual, expected interface{}, msg ...interface{}) bool {
 	t.Helper()
+	protoActual, proto1 := actual.(protoreflect.ProtoMessage)
+	protoExpected, proto2 := expected.(protoreflect.ProtoMessage)
+	if proto1 && proto2 {
+		return t.report1(actual, msg,
+			!proto.Equal(protoActual, protoExpected))
+	}
 	return t.report1(actual, msg,
-		!reflect.DeepEqual(actual, expected))
+		!deepequal.DeepEqual(actual, expected))
 }
 
 // Match checks for regex.MatchString(actual).
@@ -477,14 +530,14 @@ func (t *C) Match(actual, regex interface{}, msg ...interface{}) bool {
 
 // isMatch updates actual to be a real string used for matching, to make
 // dump easier to understand, but this result in losing type information.
-func isMatch(actual *interface{}, regex interface{}) bool {
+func isMatch(actual *interface{}, regex interface{}) bool { //nolint:gocritic // False positive.
 	if *actual == nil {
 		return false
 	}
 	if !stringify(actual) {
 		panic("actual is not a string, []byte, []rune, fmt.Stringer, error or nil")
 	}
-	s := (*actual).(string)
+	s := (*actual).(string) //nolint:forcetypeassert // False positive.
 
 	switch v := regex.(type) {
 	case *regexp.Regexp:
@@ -495,7 +548,7 @@ func isMatch(actual *interface{}, regex interface{}) bool {
 	panic("regex is not a *regexp.Regexp or string")
 }
 
-func stringify(arg *interface{}) bool {
+func stringify(arg *interface{}) bool { //nolint:gocritic // False positive.
 	switch v := (*arg).(type) {
 	case nil:
 		return false
@@ -505,10 +558,10 @@ func stringify(arg *interface{}) bool {
 		*arg = v.String()
 	default:
 		typ := reflect.TypeOf(*arg)
-		switch typ.Kind() {
+		switch typ.Kind() { //nolint:exhaustive // Covered by default case.
 		case reflect.String:
 		case reflect.Slice:
-			switch typ.Elem().Kind() {
+			switch typ.Elem().Kind() { //nolint:exhaustive // Covered by default case.
 			case reflect.Uint8, reflect.Int32:
 			default:
 				return false
@@ -550,14 +603,14 @@ func (t *C) Contains(actual, expected interface{}, msg ...interface{}) bool {
 }
 
 func isContains(actual, expected interface{}) (found bool) {
-	switch valActual := reflect.ValueOf(actual); valActual.Kind() {
+	switch valActual := reflect.ValueOf(actual); valActual.Kind() { //nolint:exhaustive // Covered by default case.
 	case reflect.String:
-		strActual := valActual.Convert(typString).Interface().(string)
+		strActual := valActual.Convert(typString).Interface().(string) //nolint:forcetypeassert // False positive.
 		valExpected := reflect.ValueOf(expected)
 		if valExpected.Kind() != reflect.String {
 			panic("expected underlying type is not a string")
 		}
-		strExpected := valExpected.Convert(typString).Interface().(string)
+		strExpected := valExpected.Convert(typString).Interface().(string) //nolint:forcetypeassert // False positive.
 		found = strings.Contains(strActual, strExpected)
 
 	case reflect.Map:
@@ -665,14 +718,21 @@ func (t *C) NotLen(actual interface{}, expected int, msg ...interface{}) bool {
 // It tries to recursively unwrap actual before checking using
 // errors.Unwrap() and github.com/pkg/errors.Cause().
 //
+// It will use proto.Equal for gRPC status errors.
+//
 // They may be a different instances, but must have same type and value.
 //
 // Checking for nil is okay, but using Nil(actual) instead is more clean.
 func (t *C) Err(actual, expected error, msg ...interface{}) bool {
 	t.Helper()
-	actual = unwrapErr(actual)
-	return t.report2(actual, expected, msg,
-		fmt.Sprintf("%#v", actual) == fmt.Sprintf("%#v", expected))
+	actual2 := unwrapErr(actual)
+	equal := fmt.Sprintf("%#v", actual2) == fmt.Sprintf("%#v", expected)
+	_, proto1 := actual2.(interface{ GRPCStatus() *status.Status })  //nolint:errorlint // False positive.
+	_, proto2 := expected.(interface{ GRPCStatus() *status.Status }) //nolint:errorlint // False positive.
+	if proto1 || proto2 {
+		equal = proto.Equal(status.Convert(actual2).Proto(), status.Convert(expected).Proto())
+	}
+	return t.report2(actual, expected, msg, equal)
 }
 
 func unwrapErr(err error) (actual error) {
@@ -680,7 +740,7 @@ func unwrapErr(err error) (actual error) {
 	actual = err
 	for {
 		actual = pkgerrors.Cause(actual)
-		wrapped, ok := actual.(interface{ Unwrap() error })
+		wrapped, ok := actual.(interface{ Unwrap() error }) //nolint:errorlint // False positive.
 		if !ok {
 			break
 		}
@@ -698,14 +758,21 @@ func unwrapErr(err error) (actual error) {
 // It tries to recursively unwrap actual before checking using
 // errors.Unwrap() and github.com/pkg/errors.Cause().
 //
+// It will use !proto.Equal for gRPC status errors.
+//
 // They must have either different types or values (or one should be nil).
 // Different instances with same type and value will be considered the
 // same error, and so is both nil.
 func (t *C) NotErr(actual, expected error, msg ...interface{}) bool {
 	t.Helper()
-	actual = unwrapErr(actual)
-	return t.report1(actual, msg,
-		fmt.Sprintf("%#v", actual) != fmt.Sprintf("%#v", expected))
+	actual2 := unwrapErr(actual)
+	notEqual := fmt.Sprintf("%#v", actual2) != fmt.Sprintf("%#v", expected)
+	_, proto1 := actual2.(interface{ GRPCStatus() *status.Status })  //nolint:errorlint // False positive.
+	_, proto2 := expected.(interface{ GRPCStatus() *status.Status }) //nolint:errorlint // False positive.
+	if proto1 || proto2 {
+		notEqual = !proto.Equal(status.Convert(actual2).Proto(), status.Convert(expected).Proto())
+	}
+	return t.report1(actual, msg, notEqual)
 }
 
 // Panic checks is actual() panics.
@@ -713,7 +780,7 @@ func (t *C) NotErr(actual, expected error, msg ...interface{}) bool {
 // It is able to detect panic(nil)… but you should try to avoid using this.
 func (t *C) Panic(actual func(), msg ...interface{}) bool {
 	t.Helper()
-	var didPanic = true
+	didPanic := true
 	func() {
 		defer func() { _ = recover() }()
 		actual()
@@ -728,7 +795,7 @@ func (t *C) Panic(actual func(), msg ...interface{}) bool {
 // It is able to detect panic(nil)… but you should try to avoid using this.
 func (t *C) NotPanic(actual func(), msg ...interface{}) bool {
 	t.Helper()
-	var didPanic = true
+	didPanic := true
 	func() {
 		defer func() { _ = recover() }()
 		actual()
@@ -746,7 +813,7 @@ func (t *C) NotPanic(actual func(), msg ...interface{}) bool {
 func (t *C) PanicMatch(actual func(), regex interface{}, msg ...interface{}) bool {
 	t.Helper()
 	var panicVal interface{}
-	var didPanic = true
+	didPanic := true
 	func() {
 		defer func() { panicVal = recover() }()
 		actual()
@@ -776,7 +843,7 @@ func (t *C) PanicMatch(actual func(), regex interface{}, msg ...interface{}) boo
 func (t *C) PanicNotMatch(actual func(), regex interface{}, msg ...interface{}) bool {
 	t.Helper()
 	var panicVal interface{}
-	var didPanic = true
+	didPanic := true
 	func() {
 		defer func() { panicVal = recover() }()
 		actual()
@@ -813,7 +880,7 @@ func (t *C) Less(actual, expected interface{}, msg ...interface{}) bool {
 }
 
 func isLess(actual, expected interface{}) bool {
-	switch v1, v2 := reflect.ValueOf(actual), reflect.ValueOf(expected); v1.Kind() {
+	switch v1, v2 := reflect.ValueOf(actual), reflect.ValueOf(expected); v1.Kind() { //nolint:exhaustive // Covered by default case.
 	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
 		return v1.Int() < v2.Int()
 	case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uintptr:
@@ -851,7 +918,7 @@ func (t *C) LessOrEqual(actual, expected interface{}, msg ...interface{}) bool {
 }
 
 func isGreater(actual, expected interface{}) bool {
-	switch v1, v2 := reflect.ValueOf(actual), reflect.ValueOf(expected); v1.Kind() {
+	switch v1, v2 := reflect.ValueOf(actual), reflect.ValueOf(expected); v1.Kind() { //nolint:exhaustive // Covered by default case.
 	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
 		return v1.Int() > v2.Int()
 	case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uintptr:
@@ -929,7 +996,7 @@ func (t *C) Between(actual, min, max interface{}, msg ...interface{}) bool {
 }
 
 func isBetween(actual, min, max interface{}) bool {
-	switch v, vmin, vmax := reflect.ValueOf(actual), reflect.ValueOf(min), reflect.ValueOf(max); v.Kind() {
+	switch v, vmin, vmax := reflect.ValueOf(actual), reflect.ValueOf(min), reflect.ValueOf(max); v.Kind() { //nolint:exhaustive // Covered by default case.
 	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
 		return vmin.Int() < v.Int() && v.Int() < vmax.Int()
 	case reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64, reflect.Uintptr:
@@ -940,7 +1007,8 @@ func isBetween(actual, min, max interface{}) bool {
 		return vmin.String() < v.String() && v.String() < vmax.String()
 	default:
 		if actualTime, ok := actual.(time.Time); ok {
-			minTime, maxTime := min.(time.Time), max.(time.Time)
+			minTime := min.(time.Time) //nolint:forcetypeassert // Want panic.
+			maxTime := max.(time.Time) //nolint:forcetypeassert // Want panic.
 			return minTime.Before(actualTime) && actualTime.Before(maxTime)
 		}
 	}
@@ -1003,7 +1071,7 @@ func (t *C) InDelta(actual, expected, delta interface{}, msg ...interface{}) boo
 }
 
 func isInDelta(actual, expected, delta interface{}) bool {
-	switch v, e, d := reflect.ValueOf(actual), reflect.ValueOf(expected), reflect.ValueOf(delta); v.Kind() {
+	switch v, e, d := reflect.ValueOf(actual), reflect.ValueOf(expected), reflect.ValueOf(delta); v.Kind() { //nolint:exhaustive // Covered by default case.
 	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
 		min, max := e.Int()-d.Int(), e.Int()+d.Int()
 		return min <= v.Int() && v.Int() <= max
@@ -1015,7 +1083,8 @@ func isInDelta(actual, expected, delta interface{}) bool {
 		return min <= v.Float() && v.Float() <= max
 	default:
 		if actualTime, ok := actual.(time.Time); ok {
-			expectedTime, dur := expected.(time.Time), delta.(time.Duration)
+			expectedTime := expected.(time.Time) //nolint:forcetypeassert // Want panic.
+			dur := delta.(time.Duration)         //nolint:forcetypeassert // Want panic.
 			minTime, maxTime := expectedTime.Add(-dur), expectedTime.Add(dur)
 			return minTime.Before(actualTime) && actualTime.Before(maxTime) ||
 				actualTime.Equal(minTime) ||
@@ -1106,7 +1175,7 @@ func (t *C) HasPrefix(actual, expected interface{}, msg ...interface{}) bool {
 
 // isHasPrefix updates actual and expected to be a real string used for check,
 // to make dump easier to understand, but this result in losing type information.
-func isHasPrefix(actual, expected *interface{}) bool {
+func isHasPrefix(actual, expected *interface{}) bool { //nolint:gocritic // False positive.
 	if *actual == nil || *expected == nil {
 		return false
 	}
@@ -1147,7 +1216,7 @@ func (t *C) HasSuffix(actual, expected interface{}, msg ...interface{}) bool {
 
 // isHasSuffix updates actual and expected to be a real string used for check,
 // to make dump easier to understand, but this result in losing type information.
-func isHasSuffix(actual, expected *interface{}) bool {
+func isHasSuffix(actual, expected *interface{}) bool { //nolint:gocritic // False positive.
 	if *actual == nil || *expected == nil {
 		return false
 	}
@@ -1215,7 +1284,7 @@ func jsonify(arg interface{}) json.RawMessage {
 		}
 		return *v
 	}
-	buf := reflect.ValueOf(arg).Convert(typBytes).Interface().([]byte)
+	buf := reflect.ValueOf(arg).Convert(typBytes).Interface().([]byte) //nolint:forcetypeassert // Want panic.
 
 	var v interface{}
 	err := json.Unmarshal(buf, &v)
