@@ -1,3 +1,4 @@
+//nolint:goerr113 // It's just a test.
 package check_test
 
 import (
@@ -13,6 +14,10 @@ import (
 	"time"
 
 	pkgerrors "github.com/pkg/errors"
+	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/status"
+	"google.golang.org/protobuf/types/known/emptypb"
+	"google.golang.org/protobuf/types/known/timestamppb"
 
 	"github.com/powerman/check"
 )
@@ -58,7 +63,7 @@ var (
 	zSlice   []int
 	zString  string
 	zStruct  struct{}
-	// zUnsafe     unsafe.Pointer // don't like to import unsafe
+	// zUnsafe     unsafe.Pointer // don't like to import unsafe.
 	zBoolPtr    *bool
 	zIntPtr     *int
 	zInt8Ptr    *int8
@@ -89,6 +94,7 @@ var (
 	zJSON     json.RawMessage
 	zJSONPtr  *json.RawMessage
 	zTime     time.Time
+	zProto    emptypb.Empty
 	// Initialized but otherwise zero-like values.
 	vChan              = make(chan int)
 	vFunc              = func() {}
@@ -118,7 +124,7 @@ var (
 	xSlice             = []int{3, 5, 8}
 	xString            = "<nil>"
 	xStruct            = myStruct{i: 10, s: "ten"}
-	// xUnsafe                      = unsafe.Pointer(&xUintptr) // don't like to import unsafe
+	// xUnsafe                      = unsafe.Pointer(&xUintptr) // don't like to import unsafe.
 	xBoolPtr    = &xBool
 	xIntPtr     = &xInt
 	xInt8Ptr    = &xInt8
@@ -141,13 +147,15 @@ var (
 	xSlicePtr   = &xSlice
 	xStringPtr  = &xString
 	xStructPtr  = &xStruct
-	// xUnsafePtr  *unsafe.Pointer  = &xUnsafe // don't like to import unsafe
+	// xUnsafePtr  *unsafe.Pointer  = &xUnsafe // don't like to import unsafe.
 	xMyInt    myInt           = 31337
 	xMyString myString        = "xyz"
 	xJSON     json.RawMessage = []byte(`{"s":"ten","i":10}`)
 	xJSONPtr                  = &xJSON
 	xTime                     = time.Now()
 	xTimeEST                  = xTime.In(func() *time.Location { loc, _ := time.LoadLocation("EST"); return loc }())
+	xProto                    = timestamppb.Now()
+	xGRPCErr                  = status.Error(codes.Unknown, "unknown")
 )
 
 func TestTODO(tt *testing.T) {
@@ -173,6 +181,22 @@ func TestTODO(tt *testing.T) {
 	t.True(false)
 }
 
+func TestError(tt *testing.T) {
+	t := check.T(tt)
+	t = t.TODO()
+	t.Error()
+	t.Error("message")
+	t.Error("format: %q", "message")
+}
+
+func TestMustAll(tt *testing.T) {
+	t := check.T(tt).MustAll()
+	t.Nil(nil)
+	t.NotNil(false)
+	t.TODO().Nil(false)
+	t.TODO().NotNil(nil)
+}
+
 func TestMust(tt *testing.T) {
 	t := check.T(tt)
 	t.Must(t.Nil(nil))
@@ -227,7 +251,7 @@ func TestCheckerNilTrue(tt *testing.T) {
 	t.True(zSlice == nil)
 	t.True(zString == "")
 	t.True(zStruct == struct{}{})
-	// t.True(zUnsafe == nil)
+	// // t.True(zUnsafe == nil)
 	t.True(zBoolPtr == nil)
 	t.True(zIntPtr == nil)
 	t.True(zInt8Ptr == nil)
@@ -251,7 +275,7 @@ func TestCheckerNilTrue(tt *testing.T) {
 	t.True(zSlicePtr == nil)
 	t.True(zStringPtr == nil)
 	t.True(zStructPtr == nil)
-	// t.True(zUnsafePtr == nil)
+	// // t.True(zUnsafePtr == nil)
 	t.True(zMyInt == 0)
 	t.True(zMyString == "")
 	t.True(zJSON == nil)
@@ -413,6 +437,7 @@ func TestCheckerEqual(tt *testing.T) {
 		{false, zJSON, xJSON},
 		{true, zJSONPtr, xJSONPtr},
 		{true, zTime, xTime},
+		{false, zProto, xProto}, //nolint:govet // This is dirty (copylocks), but it's a test.
 		{true, vChan, xChan},
 		{false, vFunc, xFunc},
 		{true, vIface, xIface},
@@ -475,10 +500,10 @@ func TestCheckerEqual(tt *testing.T) {
 	t.False(xTime == xTimeEST)
 	t.Equal(xTime, xTimeEST)
 	t.EQ(xTime, xTimeEST)
-	t.NotDeepEqual(xTime, xTimeEST)
+	t.DeepEqual(xTime, xTimeEST)
 	todo.NotEqual(xTime, xTimeEST)
 	todo.NE(xTime, xTimeEST)
-	todo.DeepEqual(xTime, xTimeEST)
+	todo.NotDeepEqual(xTime, xTimeEST)
 
 	// Equal not match or panic, DeepEqual match.
 	type notComparable struct {
@@ -494,6 +519,8 @@ func TestCheckerEqual(tt *testing.T) {
 		{true, &testing.T{}, &testing.T{}},
 		{false, []byte{2, 5}, []byte{2, 5}},
 		{false, notComparable{"a", []int{3, 5}}, notComparable{"a", []int{3, 5}}},
+		{false, zProto, zProto}, //nolint:govet // This is dirty (copylocks), but it's a test.
+		{false, xGRPCErr, xGRPCErr},
 	}
 	for _, v := range cases {
 		if v.comparable {
@@ -1261,6 +1288,11 @@ func half(v interface{}) interface{} {
 		return val.Uint() / 2
 	case reflect.Float32, reflect.Float64:
 		return val.Float() / 2
+	case reflect.Complex128, reflect.Complex64: // ???
+	// No meaningful "half":
+	case reflect.Array, reflect.Slice, reflect.Map, reflect.Struct, reflect.Bool, reflect.String:
+	case reflect.Chan, reflect.Func, reflect.Interface, reflect.Invalid:
+	case reflect.Ptr, reflect.UnsafePointer:
 	}
 	panic(fmt.Sprintf("can't get half from %#v", v))
 }
@@ -1337,7 +1369,7 @@ func TestCheckerSubstring(t *testing.T) {
 		{[]byte("Sunday"), []byte("Monday")},
 		{[]rune("Sunday"), []rune("Monday")},
 		{time.Sunday, time.Monday},
-		{errors.New("Sunday"), errors.New("Monday")}, // nolint:golint
+		{errors.New("Sunday"), errors.New("Monday")},
 	}
 
 	t.Run("HasPrefix", func(tt *testing.T) {
@@ -1635,6 +1667,9 @@ func TestCheckers(t *testing.T) {
 			{true, false, false, pkgerrors.Wrap(fmt.Errorf("wrapped: %w", io.EOF), "wrapped2"), io.EOF},
 			{true, false, false, pkgerrors.Wrap(pkgerrors.Wrap(fmt.Errorf("wrapped4: %w", fmt.Errorf("wrapped3: %w", pkgerrors.Wrap(fmt.Errorf("wrapped: %w", io.EOF), "wrapped2"))), "wrapped5"), "wrapped6"), io.EOF},
 			{false, false, false, io.EOF, &myError{"EOF"}},
+			{true, true, true, xGRPCErr, xGRPCErr},
+			{true, true, false, xGRPCErr, status.Error(codes.Unknown, "unknown")},
+			{false, false, false, xGRPCErr, nil},
 		}
 		for _, v := range cases {
 			t.Run("", func(tt *testing.T) {
@@ -1668,8 +1703,8 @@ func TestCheckers(t *testing.T) {
 		todo.Panic(func() {})
 		t.NotPanic(func() {})
 
-		t.Panic(func() { panic(nil) })
-		todo.NotPanic(func() { panic(nil) })
+		t.Panic(func() { panic(nil) })       //nolint:govet // Testing nil panic.
+		todo.NotPanic(func() { panic(nil) }) //nolint:govet // Testing nil panic.
 
 		t.Panic(func() { panic("") })
 		t.Panic(func() { panic("oops") })
@@ -1693,12 +1728,12 @@ func TestCheckers(t *testing.T) {
 		todo.PanicMatch(func() {}, `test`)
 		todo.PanicNotMatch(func() {}, `test`)
 
-		t.PanicMatch(func() { panic(nil) }, ``)
-		todo.PanicNotMatch(func() { panic(nil) }, ``)
-		t.PanicMatch(func() { panic(nil) }, `^<nil>$`)
-		todo.PanicNotMatch(func() { panic(nil) }, `^<nil>$`)
-		t.PanicNotMatch(func() { panic(nil) }, `test`)
-		todo.PanicMatch(func() { panic(nil) }, `test`)
+		t.PanicMatch(func() { panic(nil) }, ``)              //nolint:govet // Testing nil panic.
+		todo.PanicNotMatch(func() { panic(nil) }, ``)        //nolint:govet // Testing nil panic.
+		t.PanicMatch(func() { panic(nil) }, `^<nil>$`)       //nolint:govet // Testing nil panic.
+		todo.PanicNotMatch(func() { panic(nil) }, `^<nil>$`) //nolint:govet // Testing nil panic.
+		t.PanicNotMatch(func() { panic(nil) }, `test`)       //nolint:govet // Testing nil panic.
+		todo.PanicMatch(func() { panic(nil) }, `test`)       //nolint:govet // Testing nil panic.
 
 		t.PanicMatch(func() { panic("") }, regexp.MustCompile(`^$`))
 		t.PanicMatch(func() { panic("oops") }, `(?i)Oops`)
